/* hand.c	-- Yasuhiro Yamazaki Tue Oct 23 2001
 */
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include "hand.h"
#include "finger.h"
#include "cube.h"

/* Υǥ */
HandModel *
createHandModel( void )
{
  int i, j;
  HandModel *ret;
  
  /* ǥեȤͤȤƤ餫ͤꤷƤ */
  const float init_position[ 5 ][ 3 ]={
    {  4.0f, -2.5f, 0.0f },
    {  3.5f,  4.5f, 0.0f },
    {  1.4f,  4.5f, 0.0f },
    { -1.3f,  4.5f, 0.0f },
    { -3.2f,  4.5f, 0.0f }
  };
  const float init_length[ 5 ]={ 1.5f, 1.5f, 1.7f, 1.6f, 1.4f };
  const int init_joint[ 5 ]={ 2, 3, 3, 3, 3 };
  const float init_rotation_x[ 5 ][ 3 ]={
    { 20.0f, 80.0f,  0.0f },
    { 20.0f, 20.0f, 10.0f },
    { 40.0f, 40.0f, 20.0f },
    { 85.0f, 80.0f, 80.0f },
    { 85.0f, 85.0f, 85.0f }
  };
  const float init_rotation_z[ 5 ]={ -60.0f, -5.0f, 0.0f, 5.0f, 10.0f };

  ret = (HandModel *)malloc( sizeof( HandModel ) );
  ret->howmany = 5;
  ret->finger = (FingerModel **)malloc( 5 * sizeof( FingerModel ) );

  for( i=0; i<5; i++ ){
    ret->finger[i] = createFingerModel();
    setHandModelFingerPosition( ret, i, 
				init_position[i][0],
				init_position[i][1],
				init_position[i][2] );
    setHandModelFingerLength( ret, i, init_length[i] );
    setHandModelFingerNumOfJoint( ret, i, init_joint[i] );
    setHandModelFingerRotation( ret, i, init_rotation_z[i] );
    for( j=0; j<ret->finger[i]->joint; j++ ){
      setHandModelFingerJointRotation( ret, i, j, init_rotation_x[i][j] );
    }
  }
  return ret;
}

/* ΥǥѴ */
void 
destroyHandModel( HandModel *model )
{
  int i;
  for( i=0;i<model->howmany; i++ ){
    destroyFingerModel( model->finger[i] );
  }
  free( model->finger );
  free( model );
}

/* Υǥɽ */
void 
putHandModel( const HandModel *model )
{
  int i;
  
  /* ʿ褹 */
  glPushMatrix();
  glScalef( 4.5f, 4.5f, 1.0f );
  putCube();
  glPopMatrix();

  /* Ƥλؤ褹 */
  for( i=0; i<model->howmany; i++ ){
    putFingerModel( model->finger[i] );
  }
}

/* Υǥͤɽؿ */
void
printHandModel( FILE *stream, const HandModel *model )
{
  fprintf( stream, 
	   "*** HandModel %p\n", 
	   model );
}

/* Υǥؿ */
void 
setHandModelFingerPosition( HandModel *model, const int no,
			    const float x, const float y, const float z )
{
  setFingerModelPosition(  model->finger[no], x, y, z );
}
void 
setHandModelFingerLength( HandModel *model, const int no, const float length )
{
  setFingerModelLength( model->finger[no], length );
}
void 
setHandModelFingerNumOfJoint( HandModel *model, const int no, const int joint )
{
  setFingerModelNumOfJoint( model->finger[no], joint );
}
void 
setHandModelFingerJointRotation( HandModel *model, const int no,
				 const int joint, const float rotation )
{
  setFingerModelJointRotation( model->finger[no], joint, rotation );
}
void 
setHandModelFingerRotation( HandModel *model, const int no, const float rotation )
{
  setFingerModelRotation( model->finger[no], rotation );
}
void 
setHandModelNumOfFinger( HandModel *model, const int howmany )
{
  int min_size, i;
  FingerModel **finger = (FingerModel **)malloc( howmany * sizeof( FingerModel ) );
  min_size = (howmany<model->howmany)?howmany:model->howmany;
  for( i=0; i<min_size; i++ ){
    finger[i] = model->finger[i];
  }
  for( i=0; i<howmany-min_size; i++ ){
    finger[i+min_size] = createFingerModel();
  }
  for( i=0; i<model->howmany-min_size; i++ ){
    destroyFingerModel( model->finger[i+min_size] );
  }
  free( model->finger );

  model->howmany = howmany;
  model->finger = finger;
}

/* ؿ */
FingerModel *
getHandModelFinger( HandModel *model, const int no )
{
  if( no>=0 && no<model->howmany ){
    return model->finger[no];
  }
  return NULL;
}

/* ֥ǥ׻ؿ */
void 
interpolationHandModel( HandModel *dst, 
			const HandModel *src0, const HandModel *src1,
			const float rate )
{
  /*
   *  rate (0.0f-1.0f) δ֤˱֤Ǥ褦ˤƤ
   *
   * 0.0f:     *dst  *src0 Ʊͤˤʤޤ
   * 1.0f:     *dst  *src1 Ʊͤˤʤޤ
   * ʳ: *dst Ϥγ˱ *src0  *src1 δ֤ͤˤʤޤ
   *
   *  interpolationFingerModel() 礷ƹͤƲ
   */
}
